/* 
 * tclLoadAddon.c --
 *
 *	This procedure provides a version of the Tcl load command that
 *	works with the Be add-on image system.
 *
 *	By Mike Perham (mperham@cs.cornell.edu)
 *
 */

#include "tclInt.h"

#include <KernelKit.h>
#include <StorageKit.h>

extern "C" {
extern int TclLoadFile(Tcl_Interp *, char *, char *, char *, Tcl_PackageInitProc **, Tcl_PackageInitProc **);
extern int TclGuessPackageName(char *, Tcl_DString *);
};

/*
 *----------------------------------------------------------------------
 *
 * TclLoadFile --
 *
 *	Dynamically loads a binary code file into memory and returns
 *	the addresses of two procedures within that file, if they
 *	are defined.
 *
 * Results:
 *	A standard Tcl completion code.  If an error occurs, an error
 *	message is left in interp->result.  *proc1Ptr and *proc2Ptr
 *	are filled in with the addresses of the symbols given by
 *	*sym1 and *sym2, or NULL if those symbols can't be found.
 *
 * Side effects:
 *	New code suddenly appears in memory.
 *
 *----------------------------------------------------------------------
 */

int
TclLoadFile(Tcl_Interp *interp, char *fileName, char *sym1, char *sym2, 
	Tcl_PackageInitProc **proc1Ptr, Tcl_PackageInitProc **proc2Ptr)
{
    image_id handle;
    BFile file;
    record_ref ref;
    long result;

	/* fileName must be an absolute path in DR8 according to BeBook */
   	if (get_ref_for_path(fileName, &ref) < B_NO_ERROR) {
   		return TCL_ERROR;
   	}
    
    file.SetRef(ref);
    handle = load_add_on(&file);
    if (handle == B_ERROR) {
		return TCL_ERROR;
    }

    result =  get_image_symbol(handle, sym1, 
    		B_SYMBOL_TYPE_TEXT, proc1Ptr);

    if (result != B_NO_ERROR) {
		return TCL_ERROR;
    }
    
    result = get_image_symbol(handle, sym2, 
    		B_SYMBOL_TYPE_TEXT, proc2Ptr);

    if (result != B_NO_ERROR) {
		return TCL_ERROR;
    }
    return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * TclGuessPackageName --
 *
 *	If the "load" command is invoked without providing a package
 *	name, this procedure is invoked to try to figure it out.
 *
 * Results:
 *	Always returns 0 to indicate that we couldn't figure out a
 *	package name;  generic code will then try to guess the package
 *	from the file name.  A return value of 1 would have meant that
 *	we figured out the package name and put it in bufPtr.
 *
 * Side effects:
 *	None.
 *
    char *fileName;		Name of file containing package (already
				 * translated to local form if needed). 
    Tcl_DString *bufPtr;	 Initialized empty dstring.  Append
				 * package name to this if possible. 
 *----------------------------------------------------------------------
 */

int
TclGuessPackageName(char *fileName, Tcl_DString *bufPtr)
{
    return 0;
}
